//
//  ViewController.m
//  iAuth2demo
//
//  Copyright © 2018 identiv. All rights reserved.
//

#import "ViewController.h"
#import "iAuth2sdk.h"
#import "winscard.h"
#import "debug.h"

char buffer[32*1024];

@interface ViewController () {
    NSMutableString *ms;
    NSMutableString *msOutput;
}
@end


@implementation ViewController



- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.

    //self.debugView.delegate = self;
    
    self.debugView.layer.cornerRadius = 8.0f;
    self.debugView.layer.borderColor = [UIColor lightGrayColor].CGColor;
    self.debugView.layer.borderWidth = 1.0f;
    self.debugView.editable = false;
    
    self.outputView.layer.cornerRadius = 8.0f;
    self.outputView.layer.borderColor = [UIColor lightGrayColor].CGColor;
    self.outputView.layer.borderWidth = 1.0f;
    self.outputView.editable = false;
    
    ms = [[NSMutableString alloc]initWithCapacity:512];
    msOutput = [[NSMutableString alloc]initWithCapacity:512];
    self.debugView.text = @"";
    self.outputView.text = @"";
    
    
    //SetDebugLevel(LOG_LEVEL_ALL);
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)updateOutput {
    self.outputView.text = msOutput;
}

- (void)updateDebugMessage {
    unsigned count=0, offset=0;
    if (IsDebugDataAvailable()) {
        count = RetrieveDebugData(buffer, sizeof(buffer));
        while (offset < count) {
            [ms appendFormat:@"%s",buffer+offset];
            offset += strlen(buffer+offset)+1;
        }
        self.debugView.text = ms;
    }
}

- (IBAction)refreshDebugButtonClick:(id)sender {
    [self updateDebugMessage];
}
- (IBAction)clearDebugButtonClick:(id)sender {
    self.debugView.text = @"";
}


- (IBAction)testButtonClick:(id)sender {
 
    
    dispatch_queue_t queue = dispatch_get_global_queue(QOS_CLASS_USER_INTERACTIVE, 0);
    dispatch_async(queue, ^{
        [self pcscTest];
    });

}

-(void)pcscTest {
    SCARDCONTEXT context=0;
    char szReader[64];
    DWORD dwLength=0;
    LONG status;
    SCARD_READERSTATE rs;
    SCARDHANDLE cardHandle=0;
    DWORD dwProtocol;
    unsigned char testApdu[] = { 0x00, 0xA4, 0x02, 0x00, 0x02, 0x3F, 0x00 };
    unsigned char cardResponse[256];
    static int pcscTestRunning = 0;
    

    if (!pcscTestRunning) {
        pcscTestRunning = 1;

        do {
            
            status = iAuth2GetLibVersion();
            [msOutput appendFormat:@"Library Version %d.%02d.%02d.%02d\n",
             (status >> 24), (status >> 16) & 0xFF, (status >> 8) & 0xFF, status & 0xFF];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
            //  get context
            status = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &context);
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateDebugMessage];
            });
            if (SCARD_S_SUCCESS != status) {
                break;
            }
            [msOutput appendFormat:@"SCardEstablishContext success\n"];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
            
            memset(szReader, 0, sizeof(szReader));
            dwLength = sizeof(szReader);
            status = SCardListReaders(context, NULL, szReader, &dwLength);
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateDebugMessage];
            });
            if (SCARD_S_SUCCESS != status) {
                break;
            }
            [msOutput appendFormat:@"SCardListReaders success\n"];
            [msOutput appendFormat:@"Reader found %s\n", szReader];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
            memset(&rs, 0, sizeof(rs));
            rs.szReader = szReader;
            rs.dwCurrentState = SCARD_STATE_UNAWARE;
            
            status = SCardGetStatusChange(context, 10000, &rs, 1);
            if (SCARD_S_SUCCESS == status) {
                if (rs.dwEventState & SCARD_STATE_EMPTY) {
                    rs.dwCurrentState = rs.dwEventState;
                    [msOutput appendFormat:@"Waiting for card insertion\n"];
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [self updateOutput];
                    });
                    status = SCardGetStatusChange(context, INFINITE, &rs, 1);
                    if (SCARD_S_SUCCESS != status) {
                        break;
                    }
                    [msOutput appendFormat:@"SCardGetStatusChange successful - card inserted\n"];
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [self updateOutput];
                    });
                }
            }
            
            status = SCardConnect(context, szReader, SCARD_SHARE_SHARED,
                                  SCARD_PROTOCOL_T0|SCARD_PROTOCOL_T1,
                                  &cardHandle, &dwProtocol);
            if (SCARD_S_SUCCESS != status) {
                break;
            }
            [msOutput appendFormat:@"SCardConnect successful\n"];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
            rs.cbAtr = sizeof(rs.rgbAtr);
            
            status = SCardStatus(cardHandle, NULL, NULL, NULL, NULL, rs.rgbAtr, &rs.cbAtr);
            if (SCARD_S_SUCCESS != status) {
                break;
            }
            [msOutput appendFormat:@"SCardStatus successful\n"];
            [msOutput appendFormat:@"ATR: "];
            for (int i=0; i<rs.cbAtr; i++) {
                [msOutput appendFormat:@"%02X ", rs.rgbAtr[i]];
            }
            [msOutput appendFormat:@"\n"];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
            
            dwLength = sizeof(cardResponse);
            status = SCardTransmit(cardHandle, NULL, testApdu, sizeof(testApdu), NULL, cardResponse, &dwLength);
            if (SCARD_S_SUCCESS != status) {
                break;
            }
            [msOutput appendFormat:@"SCardTransmit successful\n"];
            [msOutput appendFormat:@"APDU: "];
            for (int i=0; i<sizeof(testApdu); i++) {
                [msOutput appendFormat:@"%02X ", testApdu[i]];
            }
            [msOutput appendFormat:@"\n"];
            [msOutput appendFormat:@"RESP: "];
            for (int i=0; i<dwLength; i++) {
                [msOutput appendFormat:@"%02X ", cardResponse[i]];
            }
            [msOutput appendFormat:@"\n"];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self updateOutput];
            });
            
        } while (false);
        
        if (0 != cardHandle) {
            status = SCardDisconnect(cardHandle, SCARD_UNPOWER_CARD);
            if (SCARD_S_SUCCESS == status) {
                [msOutput appendFormat:@"SCardDisconnect successful\n"];
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self updateOutput];
                });
            }
        }
        
        if (0 != context) {
            status = SCardReleaseContext(context);
            if (SCARD_S_SUCCESS == status) {
                [msOutput appendFormat:@"SCardReleaseContext successful\n"];
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self updateOutput];
                });
            }
        }
        
        [msOutput appendFormat:@"Test completed\n"];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            [self updateOutput];
            [self updateDebugMessage];
        });
        
        pcscTestRunning = 0;
    }
}

@end

